// Binary: cache/js-dbg-32-29f08848d96c-linux
// Flags: -j
//
try {
    function l() {        }
    function r() {        }
    d = 4;
    var _ = q;
} catch (e) {    }
try {
    v = new ArrayBuffer();
    s = [];
    function e() {        }
    let a;
    eval("for(var[x]=e in this){(function(){})()}");
} catch (e) {    }
