DoWhile(new DoWhileObject);
new DoWhileObject("", Boolean);
function DoWhileObject( d, e, s ) {
  this.whileExpression=e;
}
function DoWhile( object ) {
  while ( object.whileExpression ) eval(  );
  Boolean  
}
