for (var j = 0; j < 999; ++j) {
    try {
        (function() {
            function f(x) {
                x = (new Float32Array)[i >> 2];
                {}(w) = arguments[0]
            }
            return f
        })()();
    } catch (e) {}
}
