function aes(a) {
    d = a.slice()
    for (; a < 28; a++)
    d[0] = d[0] ^ undefined
}
var sjcl = {};
sjcl.bitArray = {
    concat: function (a, b) {
        return d ? a : [];
    },
    clamp: function (a, b) {
        return a
    }
};
function G(a, b, c, d, e, f) {
    var g = [],
        h = sjcl.bitArray,
        f = [];
    f = h.concat(f, c)
    if (c) g = []
    else c = h.concat([], [])
    h.concat(g, d)
    h.clamp(f, [1,2,3])
}
function decrypt(a, b, c, d, e) {
    G(a, 1, c, d, e, b)
    var g = [],
        h = sjcl.bitArray,
        f = [];
    h.concat(f, c)
    if (c) g = []
    else c = h.concat([], [])
    h.concat(g, d)
    h.concat([], c).concat.slice
}
aes(sjcl.bitArray.clamp([]));
decrypt(1, 2, 3);
