
for (var i = 0; i < 1000; ++i) {
    function isNotEmpty(obj) {
        for (var i = 0 ; i < arguments.length; i++) {
            minorgc();
            var o = arguments[i];
        }
    };
    isNotEmpty([1]);
}
