try {} catch (e) {}
try {} catch (e) {}
try {
    (function() {
        (function() {});
    })
} catch (e) {}
try {
    t;
} catch (e) {}
try {} catch (e) {}
try {
    try {
        gcparam("markStackLimit", 1);
    } catch (e) {}
} catch (e) {}
try {
    (function() {
        ""(function() {});
    })
} catch (e) {}
try {
    for (t of Number) {
        (function() {
            (function() {});
        })
    }
} catch (e) {}
