function test() {
  var a1; var a2; var a3; var a4; var a5; var a6; var a7; var a8; var a9;
  var a10; var a11; var a12; var a13; var a14; var a15; var a16; var a17;
  var a18; var a19; var a20; var a21; var a22; var a23; var a24; var a25;
  var a26; var a27; var a28; var a29; var a30; var a31; var a32; var a33;
  var a34; var a35; var a36; var a37; var a38; var a39; var a40; var a41;
  var a42; var a43; var a44; var a45; var a46; var a47; var a48;
  for ( dbg = 30; dbg >=0; dbg-- ) {}
  var a50; var a51; var a52; var a53; var a54; var a55; var a56; var a57;
  var a58; var a59; var a60; var a61; var a62; var a63; var a64; var a65;
  var a66; var a67; var a68; var a69;
  var assertEq = '';
  var a71; var a72;
  let onDebuggerStatement;
  var a74; var a75; var a76; var a77; var a78; var a79; var a80; var a81;
  var a82; var a83; var a84; var a85; var a86; var a87; var a88; var a89;
  var a90; var a91; var a92; var a93; var a94; var a95; var a96; var a97;
  var a98; var a99; var a100; var a101; var a102; var a103; var a104; var a105;
  var a106; var a107; var a108; var a109; var a110; var a111; var a112;
  if(a111 !== a2)
    var a114;
  var a115; var a116; var a117; var a120; var a121; var a122; var a123;
  var a124; var a125;
  for (var a126 = 1; a126 < ([1,2,3]).length -1; ++a126) 1;
}

test();
